import unittest
from argument_sampling.topic_similarity import *
from collections import Counter

class testSampleFiles(unittest.TestCase):
    def test_sample_files_mostl_similar_different_topics_perspectrum(self):
        df_similar_examples_different_topics = load_similar_examples("perspectrum", "test", "/bigwork/nhwpajjy/few-shot-priming/sampling-strategies/perspectrum-most-similar-different-topics.tsv")
        #k = [2, 4, 8, 16, 32, 64]
            #self.assertEqual(df_similar_examples_different_topics["k"].unique().tolist(), k)
        stance_counts = df_similar_examples_different_topics.groupby("test-id").agg({"stance":Counter})
        expected_stance_counts = {1: 8 , 0:8}
        for stance in stance_counts.stance:
            self.assertEqual(stance, expected_stance_counts)
        topic_counts = df_similar_examples_different_topics.groupby("test-id").agg({"topic":lambda l: len(set(l))})

        for topic_count in topic_counts.topic:
            self.assertGreater(topic_count, 7)


        retrieved_counts = df_similar_examples_different_topics.groupby("test-id").agg({"id":lambda l: len(set(l))})
        for retrieved_count in retrieved_counts.id:

            self.assertEqual(retrieved_count, 16)
        self.assertEqual(len(df_similar_examples_different_topics["test-id"].unique()), 2773)


    def test_sample_files_most_similar_different_topics_vast(self):
        df_similar_examples_different_topics = load_similar_examples("vast", "test", "/bigwork/nhwpajjy/few-shot-priming/sampling-strategies/vast-most-similar-different-topics.tsv", few_shot_size=24)
        #k = [2, 4, 8, 16, 32, 64]
        #self.assertEqual(df_similar_examples_different_topics["k"].unique().tolist(), k)
        stance_counts = df_similar_examples_different_topics.groupby("test-id").agg({"stance":Counter})
        expected_stance_counts = {1: 8 , 0:8, 2:8}
        for stance in stance_counts.stance:
            self.assertEqual(stance, expected_stance_counts)
        topic_counts = df_similar_examples_different_topics.groupby("test-id").agg({"topic":lambda l: len(set(l))})

        for topic_count in topic_counts.topic:
            self.assertGreater(topic_count, 7)



        retrieved_counts = df_similar_examples_different_topics.groupby("test-id").agg({"id":lambda l: len(set(l))})
        for retrieved_count in retrieved_counts.id:

            self.assertEqual(retrieved_count, 24)

        self.assertEqual(len(df_similar_examples_different_topics["test-id"].unique()), 1460)

    def test_sample_files_most_similar_different_topics(self):
        df_similar_examples_different_topics = load_similar_examples("ibmsc", "test", "/bigwork/nhwpajjy/few-shot-priming/sampling-strategies/ibmsc-most-similar-different-topics.tsv")
    #k = [2, 4, 8, 16, 32, 64]
    #self.assertEqual(df_similar_examples_different_topics["k"].unique().tolist(), k)
        stance_counts = df_similar_examples_different_topics.groupby("test-id").agg({"stance":Counter})
        expected_stance_counts = {1: 8 , 0:8}
        for stance in stance_counts.stance:
            self.assertEqual(stance, expected_stance_counts)
        topic_counts = df_similar_examples_different_topics.groupby("test-id").agg({"topic":lambda l: len(set(l))})

        for topic_count in topic_counts.topic:
            self.assertGreater(topic_count, 7)



        retrieved_counts = df_similar_examples_different_topics.groupby("test-id").agg({"id":lambda l: len(set(l))})
        for retrieved_count in retrieved_counts.id:

            self.assertEqual(retrieved_count, 16)

        self.assertEqual(len(df_similar_examples_different_topics["test-id"].unique()), 1355)



    def test_sample_files_most_similar_different_topics__majority_stance_perspectrum(self):
        df_similar_examples_different_topics_majority_stance = load_similar_examples("perspectrum", "test", "/bigwork/nhwpajjy/few-shot-priming/sampling-strategies/perspectrum-most-similar-majority-stance-different-topic.tsv")
    #k = [2, 4, 8, 16, 32, 64]
    #self.assertEqual(df_similar_examples_different_topics["k"].unique().tolist(), k)
        df_similar_examples_different_topics_majority_stance["topic-stance"] = df_similar_examples_different_topics_majority_stance.apply(lambda x: str(hash(x["topic"])) + str(x["stance"]), axis=1)
        retrieved_counts = df_similar_examples_different_topics_majority_stance.groupby("test-id").agg({"id":lambda l: len(set(l))})
        for retrieved_count in retrieved_counts.id:
    
            self.assertEqual(retrieved_count, 16)
        #self.assertEqual(len(df_similar_examples_different_topics_majority_stance["test-id"].unique()), 2773)

        stance_counts = df_similar_examples_different_topics_majority_stance.groupby("test-id").agg({"stance":Counter})
        expected_stance_counts = {1: 8 , 0:8}
        for stance in stance_counts.stance:
            if stance != expected_stance_counts:
                print(stance)
            #self.assertEqual(stance, expected_stance_counts)
        topic_counts = df_similar_examples_different_topics_majority_stance.groupby("test-id").agg({"topic":lambda l: len(set(l))})

        for topic_count in topic_counts.topic:
            self.assertGreater(topic_count, 7)
        topic_stance_counts = df_similar_examples_different_topics_majority_stance.groupby("test-id").agg({"topic-stance": lambda l: len(set(l))})
        print(topic_stance_counts)
        for topic_count in topic_stance_counts:
            self.assertEqual(topic_count, 1)



import unittest
from experiments import *
from argument_sampling.diversify import *
class testCluster(unittest.TestCase):
    def test_get_vectors(self):
        dataset = load_splits("ibmsc", "validation")
        df_training = dataset["training"].sample(10)
        embeddings, ids, topics = get_embeddings(df_training)
        self.assertTrue(len(embeddings), 10)
        self.assertTrue(type(embeddings),np.ndarray)
        self.assertTrue(embeddings.shape, (10,768))
        return embeddings, ids, topics

    def test_save(self):
        embeddings, ids, topics = self.test_get_vectors()
        save_embeddings(embeddings, ids, topics, "/home/yamen/tmp")
        self.assertTrue(os.path.exists("/home/yamen/tmp/embeddings.txt"))
        self.assertTrue(os.path.exists("/home/yamen/tmp/topics.txt"))


    def test_load(self):
        embeddings, ids, topics = self.test_get_vectors()
        save_embeddings(embeddings, ids, topics, "/home/yamen/tmp")
        loaded_embeddings, loaded_ids, loaded_topics = load_embeddings("/home/yamen/tmp")
        self.assertTrue((loaded_embeddings==embeddings).any())
        self.assertTrue((ids == loaded_ids).any())
        self.assertEqual(topics, loaded_topics)

    def test_cluster(self):
        dataset = load_splits(experiment="ibmsc")
        embeddings, ids, topics = get_embeddings(dataset["training"])
        labels, centriods = cluster(embeddings, ids,10, n_clusters=16)
        self.assertEqual(len(centriods), 160)

    def test_sample_diversify(self):

        save_diverse_examples("ibmsc", "test")
        df_shots = sample_diverse_examples("ibmsc", "test", 16)
        self.assertGreater(df_shots["topic"].nunique(),14)


from unittest import TestCase
from few_shot_priming.baseline import *

class TestBaseline(TestCase):
    def test_random_baseline(self):
        config = get_baseline_config()
        accu, pro_f1, con_f1, mac_f1 = baseline(config, offline=True, validate=True)
        print(accu)
        self.assertTrue(True)


import unittest
from experiments import *
class TestLoad(unittest.TestCase):
    def test_load(self):
        dataset = load_splits("vast")

        training_df = dataset["training"]



        #self.assertEqual(13477, dataset["training"].shape[0])


    def test_load_topic_count(self):
        topic_count_5_dataset = load_splits("vast", topic_count=5, validate=False)
        self.assertEqual(topic_count_5_dataset["training"]["topic"].nunique(), 5)
        topic_count_1000_dataset = load_splits("vast", topic_count=1000, validate=False)
        self.assertEqual(topic_count_1000_dataset["training"]["topic"].nunique(), 1000)
